# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.preview.understand.assistant import AssistantList


class Understand(Version):

    def __init__(self, domain):
        """
        Initialize the Understand version of Preview

        :returns: Understand version of Preview
        :rtype: twilio.rest.preview.understand.Understand.Understand
        """
        super(Understand, self).__init__(domain)
        self.version = 'understand'
        self._assistants = None

    @property
    def assistants(self):
        """
        :rtype: twilio.rest.preview.understand.assistant.AssistantList
        """
        if self._assistants is None:
            self._assistants = AssistantList(self)
        return self._assistants

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand>'
